import networkx as nx
import numpy as np
import graphstate
import clifford
import util

class NXGraphState(graphstate.GraphState, nx.Graph):
    """ This is GraphState with NetworkX-like abilities """
    def __init__(self, *args, **kwargs):
        graphstate.GraphState.__init__(self, *args, **kwargs)

    def layout(self):
        """ Automatically lay out the graph """
        pos = nx.spring_layout(self, dim=3, scale=np.sqrt(self.order()))
        middle = np.average(pos.values(), axis=0)
        pos = {key: value - middle for key, value in pos.items()}
        for key, (x, y, z) in pos.items():
            self.node[key]["position"] = util.xyz(x, y, z)




# Alias some stuff to make imports cleaner
from abp.graphstate import GraphState
from abp.nxgraphstate import NXGraphState
from abp.vizclient import VizClient
from abp.qi import CircuitModel

DETERMINISTIC = False



import time, atexit, json
import networkx as nx
import numpy as np
import websocket
from socket import error as socket_error
import clifford
import util
import nxgraphstate

class VizClient(object):
    def __init__(self, uri = "ws://localhost:5000"):
        self.ws = websocket.create_connection(uri, timeout=0.1)
        atexit.register(self.shutdown)

    def shutdown(self):
        """ Close the connection to the websocket """
        self.ws.close()

    def update(self, graph, delay = 0.5):
        """ Call this function when you are ready to send data to the browser """
        g = nxgraphstate.NXGraphState(graph)

        # Automatically perform layout if position is not provided
        if not all(("position" in node) for node in g.node.values()):
            g.layout()

        # Send data to browser and rate-limit
        try:
            self.ws.send(json.dumps(g.to_json(stringify=True)))
            self.ws.recv()
        except websocket._exceptions.WebSocketTimeoutException:
            print "Timed out ... you might be pushing a bit hard"
        time.sleep(delay)


